/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package org.milaifontanals;

import com.sun.jdi.connect.spi.Connection;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.util.Properties;
import org.milaifontanals.persistence.IGestorEmpresa;
import org.milaifontanals.empresa.Departament;
import org.milaifontanals.empresa.Empleat;
import org.milaifontanals.persistence.GestorEmpresaException;

/**
 * Aplicació que comprova funcionament de les capes de persistència. Necessita
 * un fitxer de propietats el nom del qual es passarà per argument i en seu
 * defecte cercarà un fitxer de nom "infoCapa.properties".<BR>
 * Ha de contenir les propietats:</BR>
 * - nomCapa, amb el nom de la capa<BR>
 * - nomFitxerConfiguracioCapa, amb el nom del fitxer que necessita la capa per
 * poder ser invocada
 *
 * @author Usuari
 */
public class ProvaCapes {

    public static void main(String[] args) {
        String nomFitxer = null;
        if (args.length == 0) {
            nomFitxer = "infoCapa.properties";
        } else {
            nomFitxer = args[0];
        }

        Properties props = new Properties();
        try {
            props.load(new FileReader(nomFitxer));
        } catch (FileNotFoundException ex) {
            System.out.println("No es troba fitxer de propietats " + nomFitxer);
            infoError(ex);
            System.out.println("Avortem aplicació");
            return;
        } catch (IOException ex) {
            System.out.println("Error en carregar fitxer de propietats " + nomFitxer);
            infoError(ex);
            System.out.println("Avortem aplicació");
            return;
        }

        String nomCapa = props.getProperty("nomCapa");
        if (nomCapa == null || nomCapa.equals("")) {
            System.out.println("Fitxer de propietats " + nomFitxer + " no conté propietat nomCapa");
            System.out.println("Avortem aplicació");
            return;
        }
        String nomFitxerConfiguracioCapa = props.getProperty("nomFitxerConfiguracioCapa");
        if (nomFitxerConfiguracioCapa == null || nomFitxerConfiguracioCapa.equals("")) {
            System.out.println("Fitxer de propietats " + nomFitxer + " no conté propietat nomFitxerConfiguracioCapa");
            System.out.println("Avortem aplicació");
            return;
        }

        IGestorEmpresa cp = null;
        try {
            Class compo = Class.forName(nomCapa);
            Constructor c = compo.getConstructor(String.class);
            cp = (IGestorEmpresa) c.newInstance(nomFitxerConfiguracioCapa);
            System.out.println("Connexió establerta");
        } catch (Exception ex) {
            System.out.println("Problema en crear la capa de persistència");
            infoError(ex);
            return;
        }

        /* Comprovació de mètode getEmpresa() */
        try {
            System.out.println("Recuperació empresa: " + cp.getEmpresa());
        } catch (GestorEmpresaException ex) {
            infoError(ex);
        }

        System.out.println("Prova de getDepartament");
        int tprova1[] = {-5, 10, 90};
        for (int codi : tprova1) {
            System.out.println("Cerquem departament de codi " + codi + ": ");
            try {
                Departament d = cp.getDepartament(codi);
                System.out.println(d);
            } catch (Exception ex) {
                System.out.println("Problema en recuperar el departament");
                infoError(ex);
            }
        }

        System.out.println("\nProva de getEmpleat");
        int tprova2[] = {-1000, 7839, 7844, 9999};
        for (int codi : tprova2) {
            System.out.println("Cerquem empleat de codi " + codi + ": ");
            try {
                Empleat e = cp.getEmpleat(codi);
                System.out.println(e);
            } catch (GestorEmpresaException ex) {
                System.out.println("Problema en recuperar l'empleat");
                infoError(ex);
            }
        }

        System.out.println("\nProva de comptarSubordinats");
        int tprova3[] = {-1000, 7839, 7844, 7902, 9999};
        for (int codi : tprova3) {
            System.out.print("Qtat subordinats d'empleat amb " + codi + ": ");
            try {
                int q = cp.comptarSubordinats(codi);
                System.out.println(q);
            } catch (GestorEmpresaException ex) {
                System.out.println("\nProblema en comptar subordinats de l'empleat" + codi);
                infoError(ex);
            }
        }

        System.out.println("\nProva de esSubordinatDirecteIndirecte");
        int tprova4[] = {7902, 7902, 7839, 7698}; // Empleats pels que comprovarem subordinats directe/indirecte
        int emp[] = {7839, 7369, 7369, 7369}; // Empleats a mirar si són subordinats directe/indirecte
        /* En situació original de la BD:
           Subordinats de 7839: 7566 - 7698 - 7782
           Subordinats de 7566: 7788 - 7902
           Subordinats de 7698: 7499 - 7521 - 7654 - 7844 - 7900
           Subordinats de 7782: 7934
           Subordinats de 7788: 7876
           Subordinats de 7902: 7369
         */
        for (int i = 0; i < tprova4.length; i++) {
            try {
                System.out.println("Empleat " + tprova4[i] + " te per subordinat dir/ind l'empleat " + emp[i] + "? " + cp.esSubordinatDirecteIndirecte(tprova4[i], emp[i]));
            } catch (GestorEmpresaException ex) {
                System.out.println("Problema en recuperar l'empleat");
                infoError(ex);
            }
        }

        System.out.println("\nProva d'eliminar empleat");
        int tprova5[] = {-1000, 7782, 7902, 7902, 7566, 7566, 7566};
        int actCap[] = {0, 0, -1, 7900, 7788, 7876, 7698};
        /* En situació original de la BD:
            El 7782 té 1 subordinat: 7934
            El 7902 té 1 subordinat: 7369
            El 7566 té subordinat 7788 (->7876): Fem 2 proves: 
                La primera intentant substituir 7566 per subordinat directe 7788
                La segona intentant substituir 7566 per subordinat indirecte 7876
                La tercera intentant substituir 7566 per 7698 que no és subordinat (directe ni indirecte) 
         */
        for (int i = 0; i < tprova5.length; i++) {
            try {
                System.out.println("Crida eliminarEmpleat(" + tprova5[i] + "," + actCap[i] + ")");
                cp.eliminarEmpleat(tprova5[i], actCap[i]);
                cp.commit();
                System.out.println("Eliminació efectuada!");
            } catch (GestorEmpresaException ex) {
                System.out.println("Problema en eliminar empleat " + tprova5[i]);
                infoError(ex);
            }
        }

        /* Feina de l'aplicació */
        try {
            cp.closeCapa();
            System.out.println("Capa de persistència tancada");
        } catch (Exception ex) {
            System.out.println("Error en tancar la capa de persistència");
            infoError(ex);
        }

    }

    private static void infoError(Throwable aux) {
        do {
            if (aux.getMessage() != null) {
                System.out.println("\t" + aux.getMessage());
            }
            aux = aux.getCause();
        } while (aux != null);
    }

}
